@@ -22,6 +22,11 @@
 #'                    The default path for repositories if not passed as an argument.\cr
 #'   \code{cacheSaveFormat} \tab "rds" \tab What save format to use; currently,
 #'                          "qs" or "rds" \cr
+#'   \code{cacheSpeed} \tab "slow" \tab either "slow" or "fast" (1 or 2). "slow" uses
+#'                        digest::digest internally, which is transferable across Operating
+#'                        systems, but much slower than fastdigest::fastdigest. So,
+#'                        if all caching is happening on a single machine, "fast" would
+#'                        be a good setting \cr
 #'   \code{conn} \tab \code{NULL} \tab Sets a specific connection to a database, e.g.,
 #'                       \code{dbConnect(drv = RSQLite::SQLite())} or
 #'                       \code{dbConnect(drv = RPostgres::Postgres()}. For remote
@@ -76,22 +81,23 @@
 #'                         are changing frequently, so any entry in the cache repository will
 #'                         be stale following changes to functions, i.e., they will likely never
 #'                         be relevant again. This will therefore keep the cache repository
-#'                         clean of stale objects. If there is ambiguity in the \code{userTags}, i.e
-#'                         they do not uniquely identify a single entry in the cacheRepo, then
-#'                         this option will default back to the non devMode behaviour to avoid
-#'                         deleting objects. This, therefore, is most useful if the user is
-#'                         using unique values for \code{userTags}\cr
+#'                         clean of stale objects. If there is ambiguity in the \code{userTags},
+#'                         i.e., they do not uniquely identify a single entry in the
+#'                         \code{cacheRepo}, then this option will default back to the non-dev-mode
+#'                         behaviour to avoid deleting objects.
+#'                         This, therefore, is most useful if the user is using unique values for
+#'                         \code{userTags}\cr
 #'   \code{useCloud} \tab Passed to \code{Cache}. Default \code{FALSE}.\cr
-#'   \code{useDBI} \tab \code{TRUE} \tab As of reproducible 0.3, the backend is now \code{DBI}
-#'                      package instead of \code{archivist}\cr
+#'   \code{useDBI} \tab \code{TRUE} \tab As of reproducible 0.3, the backend is now \pkg{DBI}
+#'                      package instead of \pkg{archivist}\cr
 #'   \code{useGDAL} \tab Passed to \code{useGDAL} in \code{projectInputs.Raster}.
 #'                   Default \code{TRUE}.\cr
 #'   \code{useMemoise} \tab \code{FALSE} \tab Used in \code{\link{Cache}}. If \code{TRUE},
-#'                   recovery of cached elements from the cacheRepo will use
+#'                   recovery of cached elements from the \code{cacheRepo} will use
 #'                   \code{memoise::memoise}. This means that the 3rd time running a function
 #'                   will be much faster than the 1st (create cache entry) or 2nd (recover
 #'                   from the SQLite database on dist). NOTE: memoised values are removed
-#'                   when the R session is restarted. \bold{This option will use more RAM} and
+#'                   when the R session is restarted. \strong{This option will use more RAM} and
 #'                   so may need to be turned off if RAM is limiting.  \code{clearCache}
 #'                   of any sort will cause all memoising to be 'forgotten'
 #'                   (\code{memoise::forget})\cr
@@ -100,7 +106,7 @@
 #'                    will be defunct. This new algorithm will make \code{Cache} less
 #'                    sensitive to minor but irrelevant changes (like changing the
 #'                    order of arguments) and will work successfully across operating
-#'                    systems (especially relevant for the new 'cloudCache' function \cr
+#'                    systems (especially relevant for the new \code{cloudCache} function \cr
 #'   \code{verbose} \tab \code{FALSE} \tab If set to \code{TRUE} then every Cache
 #'      call will show a summary of the objects being cached, their \code{object.size}
 #'      and the time it took to digest them and also the time it took to run
@@ -111,23 +117,22 @@
 #' @section Advanced:
 #' The following options are likely not needed by a user.
 #' \tabular{lcl}{
-#'   \code{cloudChecksumsFilename} \tab \code{file.path(dirname(.reproducibleTempCacheDir), "checksums.rds")}
+#'   \code{cloudChecksumsFilename} \tab \code{file.path(dirname(.reproducibleTempCacheDir()), "checksums.rds")}
 #'                      \tab Used in \code{\link{cloudCache}} \cr
 #'   \code{length} \tab \code{Inf} \tab Used in \code{\link{Cache}}, specifically to the internal
 #'                      calls to \code{\link{CacheDigest}}. This is passed to \code{digest::digest}.
 #'                      Mostly this would be changed from default \code{Inf} if the digesting is
 #'                      taking too long. Use this with caution, as some objects will have MANY
 #'                      NA values in their first MANY elements\cr
-#'   \code{useragent} \tab \code{"http://github.com/PredictiveEcology/reproducible"}
+#'   \code{useragent} \tab \code{"https://github.com/PredictiveEcology/reproducible"}
 #'           \tab User agent for downloads using this package.\cr
 #' }
 reproducibleOptions <- function() {
   list( # nolint
     reproducible.ask = TRUE,
-    reproducible.cloudChecksumsFilename = normPath(file.path(dirname(.reproducibleTempCacheDir),
-                                                    "checksums.rds")),
-    reproducible.cachePath = normPath(file.path(.reproducibleTempCacheDir)),
+    reproducible.cachePath = .reproducibleTempCacheDir(),
     reproducible.cacheSaveFormat = "rds",
+    reproducible.cacheSpeed = "slow",
     reproducible.conn = NULL,
     reproducible.destinationPath = NULL,
     reproducible.drv = RSQLite::SQLite(),
@@ -140,13 +145,14 @@ reproducibleOptions <- function() {
     reproducible.quick = FALSE,
     reproducible.showSimilar = FALSE,
     reproducible.showSimilarDepth = 3,
+    reproducible.tempPath = .reproducibleTempPath(),
     reproducible.useCache = TRUE, # override Cache function
     reproducible.useCloud = FALSE, #
     reproducible.useDBI = TRUE,
     reproducible.useGDAL = TRUE, #
     reproducible.useMemoise = FALSE, #memoise
     reproducible.useNewDigestAlgorithm = TRUE,
-    reproducible.useragent = "http://github.com/PredictiveEcology/reproducible",
+    reproducible.useragent = "https://github.com/PredictiveEcology/reproducible",
     reproducible.verbose = FALSE
   )
 }
