@@ -15,7 +15,7 @@ test_that("all exported functions have examples", {
   exFiles <- normalizePath(dir("../../man", full.names = TRUE))
   # use for loop as it keeps control at top level
   owd <- getwd()
-  tmpdir <- file.path(tempdir(), "test_Examples") %>% checkPath(create = TRUE)
+  tmpdir <- tempdir2("test_Examples") %>% checkPath(create = TRUE)
   setwd(tmpdir)
   on.exit({
     unlink(tmpdir, recursive = TRUE)
